# Web-Crawling und Web-Scraping

Jetzt, wo die nötige Software installiert ist, kannst du mit dem Datensammeln für dein Korpus loslegen. Dazu wird das Programm sich für dich im Internet bewegen - in Form eines _Web-Crawlers_. Wir benützen dazu die Python-Bibliothek Trafilatura, zu der du unten mehr lesen kannst, wenn du willst.

Ein Web-Crawler sucht, ausgehend von einer Start-URL, weitere erreichbare bzw. verlinkte URLs. Die wohl am meisten genutzten Web-Crawler sind die von Search-Engines wie Google oder DuckDuckGo, welche dafür sorgen, dass die angezeigten Suchresultate aktuell bleiben. 
Wir haben aber nicht vor, so grossflächig zu crawlen, sondern werden uns auf die Unterseiten einer URL (also z.B. die eines Blogs) beschränken, um thematisch einigermassen einheitliche Korpora zu erhalten. In anderen Worten wird die _Scope_ der Suche beschränkt. 

Wenn dein Web-Crawler nun eine Liste von URLs gefunden hat, laden wir die Texte der Websiten herunter, was man _Web-Scraping_ nennt. In unserem Fall wird nur reiner Text in XML-Format heruntergeladen, allerdings mit Kommentaren, falls sich auf der Seite welche befinden. 

Und schon hast du den Rohtext, die Basis für dein Korpus.

# Trafilatura

Trafilatura ist eine Python-Bibliothek, die zum Web-Crawling und -Scraping sowie zum Download der gefundenen Inhalte in 
gewünschtem Format geeignet ist. Sie ist Open Source und wurde von Adrien Barbaresi erstellt. 
[Hier](https://trafilatura.readthedocs.io/en/latest/index.html#) befindet sich die ausführliche Dokumentation des Tools, 
und [hier](https://github.com/adbar/trafilatura) der Link zum zugehörigen Git-Repo.

# Inhalte dieses Ordners

Dieser Ordner enthält zwei Skripte; eines zum Web-Crawlen und eines zum Web-Scrapen. 

**Achtung**: Leider kann Trafilatura nicht jede Website crawlen und/oder scrapen. Da musst du einfach ausprobieren. 

- **crawl_websites.py**
    
    Dieses Skript nimmt eine URL als Input und sucht nach weiteren URLs auf derselben Domain. Die gefundenen URLs werden in ein Output-File geschrieben. Du kannst das Skript wie folgt benützen: 

    `python3 crawl_websites.py "Start_URL" outputfile.txt`

    Wenn du ein Windows-User bist: `python3` (immer) mit `py` ersetzen.

    Dabei muss das Output-File, das du nennen kannst wie du willst, noch nicht existieren. Es wird automatisch erstellt und der Output wird hineingeschreiben. Falls du aber ein File nennst, das schon existiert, wird der Inhalt des Files überschrieben. 


- **scrape_websites.py**

    Dieses Skript nimmt ein File mit URLs, jede auf einer eigenen Zeile, als Input. Die Inhalte der Websiten werden heruntergeladen und alle im XML-Format ins gleiche Output-File geschrieben. Das Skript benützt man wie folgt: 

    `python3 scrape_websites.py file_with_urls.txt outputfile.xml`

    Für die weitere Verwendung des Outputfiles ist es wichtig, das Dateiformat .xml zu verwenden.



# Make it your own

- Willst du die **Crawl-Resultate von mehr als einer Domain im gleichen URL-File speichern**, 
um etwa dann das Skript scrape_websites.py nur einmal anwenden zu müssen, kannst du im Code 
von crawl_websites.py auf Zeile 56 das "w" in ein "a" umwandeln. Dann kannst du mehrmals crawl_websites.py 
mit demselben Outputfile und verschiedenen URL-Inputs laufen lassen. Dadurch hast du dann alle im gleichen File!


- In crawl_websites.py kann man auf Zeile 23 die beiden **Limits** für "besuchte" und "gesehene" Websiten anpassen und erhöhen,
um möglichst alle Websites zu crawlen.
Es macht grundsätzlich Sinn, etwa 10x mehr "Gesehene" als "Besuchte" zuzulassen, aber du kannst gerne verschiedene
Limits ausprobieren. 


- Das Skript crawl_websites.py findet im Default, also wenn du nichts an dem Code änderst, alle Unterseiten einer Domain.
Wenn du aber z.B. nur Artikel möchtest und die "Über uns"- oder die "Index"-Seiten schon hier **herausfiltern** willst,
kannst du den Code ab Zeile 36 entsprechend anpassen. Mit einer Regular Expression kannst du so die Struktur der 
gewünschen URLs matchen. Im Code findest du ein Beispiel dazu. Falls du deine RegEx erst an einigen Strings ausprobieren 
willst, kannst du das [auf dieser Seite](https://regex101.com) tun. Wähle dort auf der linken Seite den 
Flavor _Python_, um danach keine Anpassungen für Python mehr machen zu müssen.


- Natürlich könntest du auch einfach im Nachgang **unerwünschte URLs aus dem Outputfile löschen**. Das 
kann bei grossen Websiten u.U. etwas mühsam sein.


- In scrape_websites.py werden im Default **nur deutschsprachige Websites** heruntergeladen. Falls du das ändern
möchtest, kannst du die Flag _target_language = 'de'_ aus Zeile 48 löschen um alle Inhalte zu scrapen.

