"""
Skript um von einer URL ausgehend weitere URls derselben Domain zu finden.

Inputs:
1. eine URL als String (mit Anführungszeichen eingeben: "example.com")
2. Name der Output-Datei.

Output:
Eine Datei, die alle gefundenen URLs enthält.
Wenn die Datei bereits existiert, wird sie überschrieben!
"""

import argparse
import re
from trafilatura.spider import focused_crawler


def crawl_for_sites(url_to_crawl, output_file):
    print(f"Crawle {url_to_crawl}...")

    # Crawle URL rekursiv und suche weitere URLs (bis zum angegebenen Limit)
    to_visit, known_urls = focused_crawler(url_to_crawl, max_seen_urls=80, max_known_urls=400)

    # Wandle Daten in eine Liste und eine sortierte Liste um
    to_visit, known_urls = list(to_visit), sorted(known_urls)

    if len(to_visit) == 0 and len(known_urls) == 0:
        print(f"Es konnten ausgehend von {url_to_crawl} keine Links gefunden werden.")
        return

    # füge alle Urls in ein Set, um Duplikate zu verhindern
    url_set = set()

    for url in known_urls:
        
        # Falls du deine Links mit einer RegEx filtern willst, kannst du diesen Block entkommentieren,
        # also die drei " in Zeilen 40 und 44 löschen, dasselbe machst du auf den Zeilen 48 und 52.
        # Dann musst du die Beispiel-RegEx in Zeilen 41 und 49 an dein konkretes Beispiel anpassen,
        # und die Zeilen 45 und 53 (url_set.add(url)) auskommentieren.
        """
        matched = re.match("https://blog\.tagesanzeiger\.ch/stadtblog/\d{4}/.*", url)
        if (matched):
            url_set.add(url)
        """
        url_set.add(url)

    for url in to_visit:
        """
        matched = re.match("https://blog\.tagesanzeiger\.ch/stadtblog/\d{4}/.*", url)
        if (matched):
            url_set.add(url)
        """
        url_set.add(url)

    # Schreibe gefundene URLS ins Outputfile
    with open(output_file, "w", encoding='utf-8') as output_file_opened:
        for ele in url_set:
            output_file_opened.write(ele + "\n")

        print(f"Die URLs stehen nun in der Datei {output_file}.")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Crawl recursively for URLs")
    parser.add_argument("url_to_crawl", type=str, help="Ausgangs-URL")
    parser.add_argument("output_file", type=str, help="Name der Output-Datei, z.B. urls.txt")

    args = parser.parse_args()
    crawl_for_sites(args.url_to_crawl, args.output_file)
