"""
Skript um aus einem Trafilatura-XML-File die Kommentare passend für Promethia zu extrahieren

Input:
1. Name der Inputdatei im XML-Format, von Trafilatura gescrapt
2. Name der Outputdatei im XML-Format, kann von Promethia verarbeitet werden

Output:
XML-Datei mit den Kommentaren in passendem Format für Promethia.
Falls schon etwas in der Datei steht, wird der Inhalt per default überschrieben.
"""
import argparse
import re
#from xml_converter import check_for_tags
import lxml.etree as ET

def extract_comments(in_file, out_file):
    # Parse the XML file
    trafi_tree = ET.parse(in_file)
    # Get the root element
    trafi_root = trafi_tree.getroot()

    # Create a new ElementTree
    promethia_root = ET.ElementTree(ET.Element("corpus"))
    # Get the root element
    corpus_root = promethia_root.getroot()

    for index, doc_element in enumerate(trafi_root.iter("doc")):
        # if comment exists, make a entry
        doc_content = doc_element.find("comments")
        test_string = ET.tostring(doc_content, method="text", encoding='Unicode')
        if test_string is not '\n':
            # Create a 'text' element and add it to the 'corpus'
            text = ET.SubElement(corpus_root, "text")
            # Create a 'header' element and add it to 'text'
            header = ET.SubElement(text, "header")
            # Create 'id' and 'title' elements and add them to 'header'
            id_element = ET.SubElement(header, "id")
            id_element.text = str(index)  # Replace with your ID value

            doc_title = doc_element.get("title")

            title_element = ET.SubElement(header, "title")
            title_element.text = doc_title  # Replace with your title value

            # Create a 'body' element and add it to 'text'
            body = ET.SubElement(text, "body")
            body.text = ET.tostring(doc_content, method="text", encoding='Unicode')  # Replace with your text content
            print(f'&/{body.text}/&')

    # Save the XML to a file
    output_file_path = out_file
    promethia_root.write(
        output_file_path, pretty_print=True, xml_declaration=True, encoding="utf-8"
    )
    print(f"XML data saved to {output_file_path}")
    print("Done")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Kommentar-Extraktor")
    parser.add_argument("in_file", type=str, help="Input-Datei (als Trafilatura-XML). Enthält alles.")
    parser.add_argument("out_file", type=str, help="Output-Datei (als Promethia-XML). Enthält nur Kommentare.")
    args = parser.parse_args()
    extract_comments(args.in_file, args.out_file)
