"""
Skript um ein XML-File von Trafilaturas auf Promethias Format zu konvertieren.

Input:
1. Name der Inputdatei im XML-Format, von Trafilatura gescrapt
2. Name der Outputdatei im XML-Format

Output:
XML-Datei mit den Textdaten in passendem Format für Promethia.
"""

import argparse
import re
import lxml.etree as ET


def xml_converter(in_file, out_file):

    # Parse the XML file
    trafi_tree = ET.parse(in_file)
    # Get the root element
    trafi_root = trafi_tree.getroot()

    # Create a new ElementTree
    promethia_root = ET.ElementTree(ET.Element("corpus"))
    # Get the root element
    corpus_root = promethia_root.getroot()

    for index, doc_element in enumerate(trafi_root.iter("doc")):
        # Create 'text' element
        text = ET.SubElement(corpus_root, "text")

        # Create 'header' element
        header = ET.SubElement(text, "header")

        # Get all attributes present in the current document
        doc_attribs = doc_element.attrib.keys()

        # Loop through all attributes dynamically
        for field in doc_attribs:
            element = ET.SubElement(header, field)
            element.text = doc_element.get(field)

        # Manually add 'id' since it's not in the document attributes
        id_element = ET.SubElement(header, "id")
        id_element.text = str(index)

        # Create 'body' element
        body = ET.SubElement(text, "body")
        doc_content = doc_element.find("main")
        if doc_content is not None:
            body.text = ET.tostring(doc_content, method="text", encoding='Unicode')
            
        # Save the XML to a file
        output_file_path = out_file
        promethia_root.write(
            output_file_path, pretty_print=True, xml_declaration=True, encoding="utf-8"
        )
    print(f"XML data saved to {output_file_path}")
    print("Done")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Trafilatura-zu-Promethia XML Converter"
    )
    parser.add_argument("in_file", type=str, help="Input-Datei (als Trafilatura-XML)")
    parser.add_argument("out_file", type=str, help="Output-Datei (als Promethia-XML)")
    args = parser.parse_args()
    xml_converter(args.in_file, args.out_file)
