# Kollokationsanalyse mit kollo

Zur Kollokationsanalyse kannst du mit dem Commandlinetool _kollo_ arbeiten. Die Beschreibung des Entwicklers und Anleitung auf Englisch, aus welcher Teile dieser Anleitung entlehnt sind, findest du [hier](https://pypi.org/project/kollo/). Es ist empfehlenswert, diese ausführlichere Beschreibung ebenfalls zu studieren. 

Die Inputfiles für kollo sind vertiklaisierte Daten, also z.B. .vrt Dateien, wie Promethia sie ausgibt. 

### Installation und Ausführung

:warning: Via pip ist Stand 01.04.2025 nicht die aktuellste Version von kollo verfügbar, insb. Windows-Nutzer:innen haben damit Probleme. Verwende stattdessen das gitlab hier: https://gitlab.uzh.ch/LiRI/projects/kollo . Zur Verwendung empfehlen wir das Pythonskript ganz unten auf der Seite, welches funktioniert, wenn man es im Repo platziert und von dort startet. :warning:

Gib folgenden Befehl ein:

```python
pip install kollo
```

Weil du kollo mit pip installierst, spielt es i.d.R. keine Rolle, in welchem Ordner du dich gerade im Terminal befindest. 

Du kannst kollo auf zwei Arten starten, entweder direkt in der Commandline oder mit einem Python Skript. In beiden gibst du jeweils die Parameter an, 
welche für deine spezifische Suche relevant sind. 

Die Argumente und ihre Position im Befehl für die Commandline siehst du im folgenden Block, der auch erscheint, wenn du im Terminal  _python -m kollo -h_ eingibst und die help-message abfragst:

```markdown
usage: kollo [-h] [-l LEFT] [-r RIGHT] [-s SPAN] [-m {ll,sll,lmi,mi,mi3,ld,t,z}] [-sw STOPWORDS] [-t TARGET] [-n NUMBER] [-o OUTPUT] [-c] [-p] [-csv [CSV]] input [query]

Extract collocations from VERT formatted corpora

positional arguments:
  input                 Input file path
  query                 Optional regex to search for (i.e. to appear in all collocation results)

optional arguments:
  -h, --help                show this help message and exit
  -l LEFT, --left LEFT      Window to the left in tokens
  -r RIGHT, --right RIGHT   Window to the right in tokens
  -s SPAN, --span SPAN      XML span to use as window (e.g. s or p)
  -m {lr,sll,lmi,mi,mi3,ld,t,z}, --metric {lr,sll,lmi,mi,mi3,ld,t,z}
                            Collocation metric
  -sw STOPWORDS, --stopwords STOPWORDS
                            Path to file containing stopwords (one per line)
  -t TARGET, --target TARGET
                            Index of VERT column to be searched as node
  -n NUMBER, --number NUMBER
                            Number of top results to return (-1 will return all)
  -o OUTPUT, --output OUTPUT
                            Comma-sep index/indices of VERT column to be calculated as collocations
  -c, --case-sensitive      Do case sensitive search
  -p, --preserve            Preserve original sequential order of tokens in bigram
  -csv [CSV], --csv [CSV]
                            Output comma-separated values
```

Eine Beispielabfrage wäre also:

```python
python3 -m kollo input/inputfile.vrt "Solidar.*$" --target=0 --output=4,0 --number=20 --left=10 --right=10 --metric=mi3 --stopwords=stopwords.txt --csv
```
Hier würde also _inputfile.vrt_ im Ordner _input_ auf Kollokationen untersucht, genauer werden die Kollokationen der Spalte 4 (Lemmas in Promethiaoutput) auf die Wortformen in Spalte 0 berechnet (siehe _target_ und _output_), sofern die Wortform die Regular Expression _"Solidar.*$"_ erfüllt. Der window (_left_ und _right_) beträgt 10 vor und nach dem Wort. Die in diesem Fall gewählte statistische Metrik (_metric_) ist MI3, die Stopwörter sind im File _stopwords.txt_ zu finden. Ausserdem werden die 20 top Resultate (_number_) in csv-format ausgegeben (_csv_). 

Falls du die Resultate nicht nur im Terminal sondern direkt in eine Datei schreiben willst, gibt man nach
--csv Flag den Namen der Outputdatei an:

```python
python3 -m kollo ..... --csv outfile.csv
```

Ein Pythonskript, welches die Kookurrenz- und Kollokationsanalyse durchführt, ist wie im folgenden Beispiel aufgebaut:  

```python
from kollo import kollo
kollo("./sample.vrt",
    query="en$",
    target=0,
    output=[1,2],
    number=3,
    left=0,
    right=1,
    metric="lr",
    stopwords="stopwords.txt",
    case_sensitive=True,
    preserve=False,
    csv=False
)
```

Das Pythonskript kann dann mit dem üblichen Befehl ausgeführen:

```python
python3 name_of_script.py
```