from top2vec import Top2Vec
import re
import plotly.express as px
import pandas as pd


def main():
    # TODO pass den Dateipfad an, um dein erstelltes Modell zu laden
    model = Top2Vec.load("Dateipfad/mein/top2vec/Modell.model")

    # .get_topic_sizes() gibt eine Liste mit der Grösse der Topics im Korpus zurück, und eine mit den id's der Topics.
    topic_sizes, topic_nums = model.get_topic_sizes()

    """
    .get_topics() nimmt die Anzahl Topics des Modelles als Argument und gibt drei Listen zurück:
        1. eine Liste mit einer Liste pro Topic mit den Top 50 Worten, die das Topic ausmachen
        2. eine Liste mit einer Liste pro Topic mit der Cosinus Distanz jedes Wortes zum entsprechenden Topic-Embedding
        3. eine Liste mit den IDs der Topics
    """
    topic_words, word_scores, topic_nums = model.get_topics(model.get_num_topics())

    # TODO pass den Dateipfad zum gewuenschten csvfile an.
    csv_path = "Dateipfad/CSVfile/.csv"

    with open(csv_path, "w", encoding="utf-8") as csv_out:
        csv_out.write("topic_number,words_in_topic,number_of_documents\n")

        for topic_num in topic_nums:
            string_topic_words = re.sub(r"[\n,\[,\]]", " ", str(topic_words[topic_num]))
            #print(string_topic_words)
            csv_out.write(
                f"{topic_num},{string_topic_words},{topic_sizes[topic_num]}\n"
            )

    """
    Visualisierung:
        Balkendiagramm aus den Topics und ihrer Anzahl. 
        Anpassbar sind  der Titel und die Beschriftungen der Achsen in .update_layout,
                        die Groesse des Diagramms in .update_layout,
                        die Breite der Balken in .update_traces,
                        die Anzahl angezeigter Worte pro Topic, momentan 7.
    """
    topic_df = pd.read_csv(csv_path)
    
    # TODO passe die Anzahl der angezeigten Worte pro Topic an, momentan 7, falls gewünscht
    shortened_words = topic_df["words_in_topic"].str.split().str[:7].apply(" ".join)
    short_words_with_ind = []
    for ind, words in enumerate(shortened_words):
        new_words = words + " :" + str(ind)
        short_words_with_ind.append(new_words)

    fig = px.bar(
        topic_df,
        x=topic_df["number_of_documents"],
        y=short_words_with_ind,
        orientation="h",
        color="number_of_documents",
    )

    # TODO passe den Titel und evtl. die Achsenbeschriftungen an
    fig.update_layout(
        title="Balkendiagramm: Beispieltitel",
        xaxis_title="Topics",
        yaxis_title="Anzahl Dokumente",
        yaxis={"categoryorder": "total ascending"},
    )
    # TODO passe die Breite der Balken und die Groesse des Diagramms an, falls gewünscht
    fig.update_traces(width=0.85)
    fig.update_layout(width=1400, height=1200)
    fig.show()


if __name__ == "__main__":
    main()
