# TWEC: Temporal Word Embeddings with a Compass

TWEC steht für **T**emporal **W**ord **E**mbeddings with a **C**ompass. Dem Ansatz liegt die Annahme zu Grunde, dass sich Wörter in ihren Bedeutungen im Laufe der Zeit verändern. Diese semantischen Wandelprozesse sollen nun in den Vektoren des Word Embedding Modells sichtbar gemacht werden können. Es wird ein Vektor für jedes Wort zu einem bestimmten Zeitpunkt erstellt, um so zeitbezogene Aussagen über die Position der einzelnen Wörter im Vektorraum treffen zu können. Dazu muss also der Textkorpus in zeitliche Abschnitte, sogenannte Slices, unterteilt werden (z.B. ein Subkorpus für das Jahr 1990, ein zweites für das Jahr 1991 usw.). Diese Slices und die sich darin befindenden Wortvektoren können dann miteinander verglichen werden. Damit die Subkorpora vergleichbar werden und übereinandergelegt werden können, braucht es einen sogenannten Kompass, der als Referenzpunkt dient. Wie dieser Kompass genau funktioniert, kann in dem [Paper der Autoren](https://ojs.aaai.org/index.php/AAAI/article/view/4594) oder in dem von Federico Bianchi veröffentlichten [Blogbeitrag](https://fede-bianchi.medium.com/aligning-temporal-diachronic-word-embeddings-with-a-compass-732ab7427955) nachgelesen werden. In seinem Blogpost beschreibt Bianchi beispielhaft die Veränderungen der Wörter "apple" und "amazon" über den Zeitraum der letzten dreissig Jahre. So wurde "amazon" um 1993 eher in der Nähe zu Wortvektoren wie "jungle", "wooden", "forests" positioniert. Im Jahr 2016 befindet es sich dagegen näher an Wörtern wie "netflix", "customer" oder "content". Durch TWEC wird es also möglich die Veränderung vom Regenwald Amazon hin zum Techgiganten Amazon zu modellieren.

Die im folgenden dargestellte Methode für das Trainieren von Word Embeddings wurde in dem 2019 erschienenen Paper ["Training Temporal Word Embeddings with a Compass"](https://ojs.aaai.org/index.php/AAAI/article/view/4594) von Valerio Di Carlo, Federico Bianchi und Matteo Palmonari begründet. Bei den folgenden Beschreibungen zur Installation von TWEC handelt es sich um eine Übersetzung der (von den Autoren zur Verfügung gestellten) Anleitung zur Installation von TWEC, siehe [hier](https://github.com/valedica/twec) für das Original. TWEC basiert auf dem word2vec Ansatz und die Word Embeddings werden darin mit dem CBOW (Continuous Bag-of-Words) Modell berechnet.

## Installation TWEC



**Wichtig**: Um diese Methode verwenden zu können, muss zuerst auf deinem Computer eine virtuelle Umgebung (virtual environment) erstellt werden. Dies liegt daran, dass TWEC eine personalisierte Version der Python-Bibliothek Gensim verwendet.

- In einem ersten Schritt muss nun das (dem Original leicht angepasste) [Repository](https://github.com/shuber9/twec_KoDuP) geclont werden. Konkret: Klick auf "<> Code" und dann unter "Clone" den HTTPS-Link kopieren, dann im gewünschten Ordner "git clone https-link" ausführen.
Am besten clonst du das Repo in denselben Ordner, in dem auch der KoDuP Ordner liegt (z.B. auf den Desktop). 
Das Clonen mit Sublime funktioniert gleich wie [hier](1._Software-Installation/Windows.md) zur Installation von KoDuP erklärt wurde.
- Nun gehst du im Terminal/in der PowerShell in den Ordner, wo du das Repo twec_KoDuP hingeclont hast, und führst folgende Befehle zur Installation aus: 
- `python3 -m venv twec_venv`

Aktivieren des venv für Mac/Linux:
- `source twec_venv/bin/activate`

Aktivieren für Windows-User:
- `.\twec_venv\Scripts\activate`

falls das nicht funktioniert, muss bei Windows möglicherweise folgender Befehl zuerst eingegeben werden:
- `Set-ExecutionPolicy RemoteSigned`


> **Windows Users**: Um TWEC auf Windows installieren und benützen zu können, sind drei zusätzliche Installationen nötig. 
>
> 1. Git muss von der cmd/PowerShell aus benutzbar sein. Installiere deshalb von [dieser Seite](https://git-scm.com/download/win) die neuste 64-Bit Version, [das](https://github.com/git-for-windows/git/releases/download/v2.40.1.windows.1/Git-2.40.1-64-bit.exe) ist der direkte Download-Link. Durch die Setup-Schritte kannst du dich mit der Default-Auswahl durchklicken.
> Um zu überprüfen, ob git installiert wurde, gibst du den Befehl `git --help`
> in der PowerShell oder der cmd ein. Erscheint Information zur Verwendung von _git_, hat die Installation geklappt. 
> 
> Falls du die folgenden Installationen nicht bereits für top2vec durchgeführt hast:
>
> 2. installiere die python developer tools: `py -m pip install python-dev-tools --user --upgrade`
>
> 3. Die Build tools von VisualStudio sind nötig, um einige der folgenden Software zu Installieren. Das etwas grössere Paket kannst du [direkt mit diesem Link](https://visualstudio.microsoft.com/de/thank-you-downloading-visual-studio/?sku=Community&channel=Release&version=VS2022&source=VSLandingPage&cid=2030&passive=false) herunterladen. 
 
Nun folgen einige Software-Installationen:
- `pip install cython`
- `pip install git+https://github.com/shuber9/gensim_for_twec`

Jetzt navigierst du in den Ordner twec_KoDuP
-  `cd twec_KoDuP/`
- `pip install -e .`

## Inhalte dieses Ordners

Dieser Ordner enthält die Skripte:

- **compute_twec_models.py**
    Dieses Skript erstellt die drei Modelle; den Kompass wie die zwei darauf beruhenden Subkorpora.
    In dem heruntergeladenen Repository befinden sich die Beispieldaten: 'examples/training/compass_parlament.txt', 'examples/training/parlament_2006.txt' und 'examples/training/parlament_2016.txt'. Du kannst das Training zuerst mit diesen Beispieldaten durchführen oder deine eigenen Daten verwenden. Um mit deinen eigenen Daten zu arbeiten, musst du im Pythonskript die Pfade und Dokumentnamen anpassen.
    
    `python3 compute_twec_models.py`


- **compare_position.py**
    In diesem Skript kannst du Vektoren in den drei Vektorräumen miteinander vergleichen.
    Im Output erhältst du daraufhin eine Liste für jedes Subkorpus, bei den Beispieldaten also eine Liste der 10 nächsten Nachbarn für das "parlament_2006_model" und das "parlament_2016_model". Du kannst diese Parameter natürlich anpassen und dir z.B. anstatt topn=10 auch topn=20 anzeigen lassen.
    
    `python3 compare_position.py`

**Achtung:** die Arbeit mit TWEC geschieht im Repository twec_KoDuP, nicht in KoDuP selbst. Die beiden Skripte befinden sich bereits im anderen Repo, hier sind sie noch zusätzlich abgelegt. 
Im Repository befindet sich zudem eine Anleitung der Autoren auf Englisch.

## Vorbereitung der Daten:
TWEC verarbeitet das gleiche Format wie das Skript zum Topic Modeling. Entsprechend kannst du die mit Promethia vertikalisierten Daten auch
für dieses Tool mit dem Skript [vrt2docperline.py](4._Korpusanalyse/Topic_Modeling/vrt2docperline.py), nach Wunsch lemmatisiert,
zum Dokument pro Zeile konvertieren lassen. 


## Quellen: 
- Bianchi, Federico (2019). **Aligning Temporal Word Embeddings with a Compass (with Code).** Erschienen au Medium, URL: https://fede-bianchi.medium.com/aligning-temporal-diachronic-word-embeddings-with-a-compass-732ab7427955, letzter Zugriff 27.02.23.
- Di Carlo, V., Bianchi, F., & Palmonari, M. (2019). **Training Temporal Word Embeddings with a Compass**. Proceedings of the AAAI Conference on Artificial Intelligence, 33(01), 6326-6334. DOI: [https://doi.org/10.1609/aaai.v33i01.33016326](https://doi.org/10.1609/aaai.v33i01.33016326)
- GitHub-Anleitung von Valerio Di Carlo für die Installation und Nutzung von TWEC: https://github.com/valedica/twec, letzter Zugriff am 27.02.23.
