"""
Skript um ein Word-Embedding Modell von einem Korpus zu erstellen.

Quelle und weiterführende Informationen:
https://radimrehurek.com/gensim/models/word2vec.html

Input:
	.vrt Datei, welche ein lemmatisiertes Dokument pro Zeile enthält

Output:
	.word_vectors Datei, welche die Vektoren eines Modells enthält
"""

import argparse
from gensim.models import Word2Vec


def create_we_model(infile):

	# in diesem Block kannst du ein neues Modell erstellen und speichern
	data_lemmatized = []
	with open(infile, "r", encoding="utf8") as in_file_open:
		# liest die einzelnen Zeilen ein und entfernt Leerzeichen
		lines = in_file_open.readlines()
		for line in lines:
			line_splitted = line.strip().split(" ")
			data_lemmatized.append(line_splitted)
	print("calculating models now")

	# Diese Parameter sind anpassbar. Insbesondere kann es sinnvoll sein, den min_count nicht auf 1 zu setzen,
	# um eine gewisse Grundlage für die Berechnungen der Vektoren sicherzustellen.
	model = Word2Vec(sentences= data_lemmatized, vector_size=300, window=5, min_count=10, workers=4)

	# Man kann das Modell als Vektoren speichern, wenn sie nicht mehr weiter trainiert werden sollen.
	word_vectors = model.wv

	# die Flag binary = False speichert die Vektoren im txt Format
	name_output=infile.split('.')[0]+".word_vectors"
	word_vectors.save_word2vec_format(name_output, binary = False)

if __name__ == "__main__":
	parser = argparse.ArgumentParser(description="Erstellt und speichert ein Word Embeddings-Modell im .txt Format")
	parser.add_argument("in_file", type=str, help="Input-Datei: Lemmatisierte Dokumente im .vrt Format")
	args = parser.parse_args()
	create_we_model(args.in_file)
