"""
Skript um Grundoperationen mit Word Embeddings kennenzulernen.

Quelle und weiterführende Informationen:
https://radimrehurek.com/gensim/models/word2vec.html

Input:
	.word_vectors Datei, welche z.B. mit dem Skript create_we_model.py erstellt wurde.
	Das .txt Format ist default, falls man ein binary-File verwenden möchte, muss die
	Flag in Zeile 21 angepasst werden.

Output:
	Informationen in der Command Line / im Terminal
"""

import argparse
from gensim.models import KeyedVectors


def we_basics(infile):
    model = KeyedVectors.load_word2vec_format(infile, binary=False)

    # In diesem Block kann man ähnliche Vektoren zu
    # einem gegebenen token ausgeben, hier z.B. die 10 nächsten zum
    # Vektor des Wortes 'Schule'
    sims = model.most_similar('Schule', topn=10)
    print('Ähnliche Vektoren zu "Schule":')
    print(sims)

    """
    # In diesem Block kann man mit den Word-Embeddings Vektorberechnungen durchführen.
    # Also zum Beispiel die Vektoren von 'Frau' und 'König' addieren, und danach 'Mann' abziehen.
    # Danach wird das Wort des passendsten Vektor ausgegeben. Klappt natürlich nur mit Vektoren, die 
    # im Text vorkommen, und ist erst ab einer gewissen Korpusgrösse sinnvoll.
    result = model.most_similar_cosmul(positive = ['Frau', 'König'], negative = ['Mann'])
    most_similar_key, similarity = result[0]  # look at the first match
    print(f"{most_similar_key}: {similarity:.4f}")
    """

    """
    # Auch die Ähnlichkeit zweier Vektoren kann man berechnen. 
    similarity = model.similarity('Hund', 'Katze')
    print(similarity)
    """


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Enthält Bausteine für einfache Operationen mit Word Embeddings")
    parser.add_argument("in_file", type=str, help="Input-Datei: Word Embeddings Modell im word_vectors Format")
    args = parser.parse_args()
    we_basics(args.in_file)
