"""
Skript, um Vektoren aus einem word_vectors Format in ein tsv-Format umzuschreiben.

Input:
    - Datei im word_vectors Format
    - Name der .tsv-Datei, die die Vektoren enthalten soll
    - Name der .tsv-Datei, die die MEtadaten zu den Vektoren enthalten soll

Output:
    - .tsv-Datei, die die Vektoren enthalten soll
    - .tsv-Datei, die die Metadaten zu den Vektoren enthalten soll
"""

import argparse
import csv

def wvec2tsv(infile, outf_vecs, outf_meta):
    with open(infile, "r", encoding='utf-8') as vec_file:
        with open(outf_vecs, "w", encoding='utf-8') as vec_tsv:
            with open(outf_meta, "w", encoding='utf-8') as meta_tsv:
                tsv_writer1 = csv.writer(vec_tsv, delimiter='\t')
                tsv_writer2 = csv.writer(meta_tsv, delimiter='\t')
                # erste Linie überspringen, sie ist ohne relevanten Inhalt
                line = vec_file.readline()
                line = vec_file.readline()
                currvec = line.strip().split(' ')
                counter = 0
                while line:
                    counter += 1
                    if currvec[0] == "":
                        break
                    tsv_writer2.writerow([currvec[0]])
                    currvec.pop(0)
                    tsv_writer1.writerow(currvec)
                    currvec = vec_file.readline().strip().split(' ')
                print("Done!")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Konvertiert .word_vectors format zu .tsv")
    parser.add_argument("infile", type=str, help="Input-Datei: Datei im word_vectors Format")
    parser.add_argument("outf_vecs", type=str, help="Output-Datei: enthält die Vektoren der Word-Embeddings")
    parser.add_argument("outf_meta", type=str, help="Output-Datei: enthält die Metadaten zu den Vektoren der Word-Embeddings")
    args = parser.parse_args()
    wvec2tsv(args.infile, args.outf_vecs, args.outf_meta)