# KoDuP Germanistik: Korpuslinguistisches Denken und Programmieren im Kontext der Germanistik: eine Lernplattform

Herzlich willkommen auf der KoDuP-Plattform am [Lehrstuhl von Noah Bubenhofer](https://www.ds.uzh.ch/l/bubenhofer)! Diese Plattform dient dazu, Studierenden der Linguistik eine Einführung ins Programmieren für die Korpuslinguistik zu geben. 

- Du willst nicht mehr an die beschränkten Suchmöglichkeiten bestehender Korpusdatenbanken gebunden sein?
- Du möchtest eigene Korpora erstellen und annotieren?
- Du möchtest über Konkordanzen und Kollokationen hinausgehende Analysemethoden anwenden?
- Du kannst zwar noch nicht programmieren, hast aber Bock, mehr mit dem Computer zu machen als Fenster rumzuschieben und dich im Textverarbeitungsprogramm rumzuärgern?

Dann bist du hier richtig. Hier erwartet dich eine nette und freundliche Einführung in die Programmierung von Python – und zwar so weit, dass du selber Korpora erstellen, annotieren und analysieren kannst.

![alt text](1._Software-Installation/screenshots/Com_M24-0204-0004.png "ETH-Rechenzentrum Clausiusstrasse, Zürich")

*Bild: ETH-Bibliothek Zürich, Bildarchiv / Fotograf: Comet Photo AG (Zürich) / Com_M24-0204-0004 / CC BY-SA 4.0*

Die Plattform ist im Aufbau und wird Semester für Semester wachsen.

## Inhalt

Der Inhalt gliedert sich in vier grosse Blöcke:

1. [Python und weitere Software installieren](1._Software-Installation/)
2. [Web-Crawling:](2._Web-Crawling/) automatisiert Daten aus dem Web herunterladen
3. [Korpuslinguistische Aufbereitung:](3._Tokenization_und_PoS_Tagging/) Tokenisierung und automatische Annotation
4. [Beispiele für Korpusanalysen mit verschiedenen Methoden](4._Korpusanalyse/)
5. [Zugriff DeReKo mit Python](5._Zugriff_DeReKo)

Hier ist eine Übersicht im Detail:

1. Installationsanleitungen und Grundlegendes: 
* [Anleitung für Windows](1._Software-Installation/Windows.md): Installation von Python, Sublime Text und Sublime Merge, SpaCy, Trafilatura und Promethia 
* [Anleitung für macOs](1._Software-Installation/macOS.md): Installation von Python, Sublime Text und Sublime Merge, SpaCy, Trafilatura und Promethia 
* [Crashkurs](1._Software-Installation/Terminal_Crashkurs.md) zur Bedienung des Terminals / der Command Line

2. Web-Crawling:
* Skript [**crawl_websites.py**](2._Web-Crawling/crawl_websites.py)
* Skript [**scrape_websites.py**](2._Web-Crawling/scrape_websites.py)

3. Korpuslinguistische Aufbereitung:
* Skript [**xml_konverter.py**](3._Tokenization_und_PoS_Tagging/xml_converter.py)
* Skript [**extract_comments.py**](3._Tokenization_und_PoS_Tagging/extract_comments.py)

4. Korpusanalyse: 
* Kookurrenzen und Kollokationen:
  * Anleitung zu [**kollo**](4._Korpusanalyse/Kookurrenzen_Kollokationen/README.md)
* Netzwerkvisualisierungen:
  * Skript [**annotate_textfiles.py**](4._Korpusanalyse/Netzwerkvisualisierungen/annotate_textfiles.py)
  * Skript [**netzwerkvis.py**](4._Korpusanalyse/Netzwerkvisualisierungen/netzwerkvis.py)
* Simple Counter: 
  * Skript [**simple_counter.py**](4._Korpusanalyse/Simple_Counter/simple_counter.py)
  * Skript [**simple_counter_v2.py**](4._Korpusanalyse/Simple_Counter/simple_counter_v2.py)
* Word Embeddings: 
  * Skript [**create_we_model.py**](4._Korpusanalyse/Word_Embeddings/create_we_model.py)
  * Skript [**we_basics.py**](4._Korpusanalyse/Word_Embeddings/we_basics.py)
  * Skript [**we_pca_vis.py**](4._Korpusanalyse/Word_Embeddings/we_pca_vis.py)
  * Anleitung [**Visualisierung mit tensorflow**](4._Korpusanalyse/Word_Embeddings/Visualisierung_mit_tensorflow.md)
  * Skript [**wvec2tsv.py**](4._Korpusanalyse/Word_Embeddings/wvec2tsv.py)
  * Anleitung zu [**TWEC**](4._Korpusanalyse/Word_Embeddings/TWEC) (**T**emporal **W**ord **E**mbeddings with a **C**ompass)
* Topic Modelling: gensim LDA-Modell oder top2vec
  * Skript [**topic_modeling.py**](4._Korpusanalyse/Topic_Modeling/topic_modeling.py)
  * Skript [**vrt2docperline.py**](4._Korpusanalyse/Topic_Modeling/vrt2docperline.py)
  * Ordner [**top2vec**](4._Korpusanalyse/Topic_Modeling/top2vec/)

5. Zugriff DeReKo:
* Skript [**KorAp_test.py**](5._Zugriff_DeReKo/KorAp_test.py)

## Credits

- Konzeption: Daniel Knuchel und Noah Bubenhofer
- Umsetzung: Niclas Bodenmann, Sonja Huber, Xenia Bojarski, Sophia Conrad, Anna Janka
- Finanzierung: kompetitiver Lehrkredit, Prorektorat Lehre und Studium, Universität


Dieses Werk ist lizenziert unter einer [http://creativecommons.org/licenses/by-nc/4.0/](Creative Commons Namensnennung-Nicht kommerziell 4.0 International Lizenz).


![alt text](https://i.creativecommons.org/l/by-nc/4.0/88x31.png "Creative Commons Lizenzvertrag")
